@@ -28,6 +28,8 @@
 #include "../libAfterStep/mystyle.h"
 #include "../libAfterStep/screen.h"
 #include "../libAfterStep/mylook.h"
+#include "../libAfterStep/session.h"
+#include "../libAfterStep/freestor.h"
 
 #include "afterconf.h"
 
@@ -109,6 +111,7 @@ TermDef       PlacementStrategyTerms[] =
     {TF_NO_MYNAME_PREPENDING, "RandomPlacement", 15, TT_FLAG, 	FEEL_RandomPlacement_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "Tile", 5, TT_FLAG, 				FEEL_Tile_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "Cascade", 7, TT_FLAG, 			FEEL_Cascade_ID, NULL},
+    {TF_NO_MYNAME_PREPENDING, "UnderPointer", 12, TT_FLAG, 			FEEL_UnderPointer_ID, NULL},
     {TF_NO_MYNAME_PREPENDING, "Manual", 6, TT_FLAG, 			FEEL_Manual_ID, NULL},
     {0, NULL, 0, 0, 0}
 };
@@ -284,21 +287,9 @@ SyntaxDef includeSyntax =
 	{TF_NO_MYNAME_PREPENDING, "DontAnimateBackground", 21, TT_INTEGER, LOOK_DontAnimateBackground_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "DontDrawBackground", 18, TT_FLAG, LOOK_DontDrawBackground_ID, NULL}
 
+#define AFTERSTEP_BALLOON_TERMS 	TITLE_BALLOON_TERMS, MENU_BALLOON_TERMS
+
 #define AFTERSTEP_TITLEBUTTON_TERMS \
-    {TF_NO_MYNAME_PREPENDING, "Balloons", 8, TT_FLAG, BALLOON_USED_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloons", 19, TT_FLAG, BALLOON_USED_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "BalloonBorderHilite", 18, TT_FLAG, BALLOON_BorderHilite_ID, &BevelSyntax}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloonBorderHilite", 29, TT_FLAG, BALLOON_BorderHilite_ID, &BevelSyntax}, \
-    {TF_NO_MYNAME_PREPENDING, "BalloonXOffset", 14, TT_INTEGER, BALLOON_XOffset_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloonXOffset", 25, TT_INTEGER, BALLOON_XOffset_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "BalloonYOffset", 14, TT_INTEGER, BALLOON_YOffset_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloonYOffset", 25, TT_INTEGER, BALLOON_YOffset_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "BalloonDelay", 12, TT_UINTEGER, BALLOON_Delay_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloonDelay", 23, TT_UINTEGER, BALLOON_Delay_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "BalloonCloseDelay", 17, TT_UINTEGER, BALLOON_CloseDelay_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloonCloseDelay", 28, TT_UINTEGER, BALLOON_CloseDelay_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "BalloonStyle", 12, TT_QUOTED_TEXT, BALLOON_Style_ID, NULL}, \
-    {TF_NO_MYNAME_PREPENDING, "TitleButtonBalloonStyle", 23, TT_QUOTED_TEXT, BALLOON_Style_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "TitleButtonSpacingLeft", 22, TT_INTEGER, LOOK_TitleButtonSpacingLeft_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "TitleButtonSpacingRight", 23, TT_INTEGER, LOOK_TitleButtonSpacingRight_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "TitleButtonSpacing", 18, TT_INTEGER, LOOK_TitleButtonSpacing_ID, NULL}, \
@@ -343,6 +334,7 @@ SyntaxDef includeSyntax =
   {TF_NO_MYNAME_PREPENDING, "SaveUnders",10             , TT_FLAG       , FEEL_SaveUnders_ID            , NULL}, \
   {TF_NO_MYNAME_PREPENDING, "PagingDefault",13          , TT_FLAG       , FEEL_PagingDefault_ID         , NULL}, \
   {TF_NO_MYNAME_PREPENDING, "AutoTabThroughDesks",19    , TT_FLAG       , FEEL_AutoTabThroughDesks_ID   , NULL}, \
+  {TF_NO_MYNAME_PREPENDING, "WarpPointer",11	    	, TT_FLAG       , FEEL_WarpPointer_ID   	, NULL}, \
   {TF_NO_MYNAME_PREPENDING, "ClickTime",9               , TT_UINTEGER   , FEEL_ClickTime_ID         	, NULL}, \
   {TF_NO_MYNAME_PREPENDING, "OpaqueMove",10             , TT_UINTEGER   , FEEL_OpaqueMove_ID        	, NULL}, \
   {TF_NO_MYNAME_PREPENDING, "OpaqueResize",12           , TT_UINTEGER   , FEEL_OpaqueResize_ID      	, NULL}, \
@@ -417,6 +409,7 @@ TermDef       LookTerms[] = {
 	INCLUDE_MYSTYLE,
 	INCLUDE_MYFRAME,
     AFTERSTEP_MYBACK_TERMS,
+	AFTERSTEP_BALLOON_TERMS,
 	AFTERSTEP_TITLEBUTTON_TERMS,
     
 	{0, NULL, 0, 0, 0}
@@ -498,6 +491,24 @@ TermDef FunctionTerms[] =
 extern TermDef       WharfTerms[];
 extern TermDef       WharfFolderTerms[];
 
+TermDef AfterStepTerms[] = 
+{
+	POPUP_TERM,
+	FUNCTION_TERM,
+	AFTERSTEP_FEEL_TERMS,
+	AFTERSTEP_CURSOR_TERMS, 
+	AFTERSTEP_MOUSE_TERMS, 
+	AFTERSTEP_KEYBOARD_TERMS, 
+	AFTERSTEP_WINDOWBOX_TERMS,
+	AFTERSTEP_LOOK_TERMS,
+	/* including MyStyles definitions processing */
+	INCLUDE_MYSTYLE,
+	INCLUDE_MYFRAME,
+    AFTERSTEP_MYBACK_TERMS,
+	AFTERSTEP_BALLOON_TERMS,
+	AFTERSTEP_TITLEBUTTON_TERMS,
+  	{0, NULL, 0, 0, 0}
+};
 /**************************************************************************
  * Syntaxes : 
  *************************************************************************/
@@ -508,6 +519,8 @@ SyntaxDef PopupSyntax 		={ '\n', '\0', PopupTerms,		0,	' ', "", "\t", "AfterStep
 SyntaxDef AutoExecSyntax 	={ '\n', '\0', FunctionTerms,	0, 	' ', "", "\t", "AfterStep Autoexec (startup/restart sequences)", "AutoExec", "functions to be executed by AfterStep on startup/shutdown", NULL, 0};
 SyntaxDef ThemeSyntax 		={ '\n', '\0', FunctionTerms,	0,	' ', "", "\t", "theme installation script", "Theme", "AfterStep theme file", NULL, 0};
 
+SyntaxDef AfterStepSyntax 		={ '\n', '\0', AfterStepTerms, 			0, '\t', "", "\t", "AfterStep proper", "Look/Feel", "AfterStep proper", NULL, 0};
+
 SyntaxDef AfterStepLookSyntax 		={ '\n', '\0', AfterStepLookTerms, 			0, '\t', "", "\t", "Look", "Look", "AfterStep look", NULL, 0};
 SyntaxDef ModuleMyStyleSyntax 		={ '\n', '\0', ModuleMyStyleTerms, 			0, '\t', "", "\t", "MyStyles", "MyStyles", "AfterStep MyStyle definitions", NULL, 0};
 SyntaxDef AfterStepMyFrameSyntax   	={ '\n', '\0', AfterStepMyFrameTerms, 		0, '\t', "", "\t", "MyFrames", "MyFrames", "AfterStep MyFrame definitions", NULL, 0};
@@ -547,5 +560,407 @@ void LinkAfterStepConfig()
 		}	 
 		++i ;	
 	}	 
+
+	pFuncSyntax->terms[F_Folder].sub_syntax = &WharfFolderSyntax ;
 }	 
 
+void init_asmodule_config( ASModuleConfig *config, Bool free_resources )
+{
+	if( config ) 
+	{
+		if( free_resources ) 
+		{
+			if( config->style_defs )
+				DestroyMyStyleDefinitions (&(config->style_defs));
+    		if( config->balloon_configs )
+			{
+				int i ;
+				for( i = 0 ; config->balloon_configs[i] != NULL ; ++i )
+				    Destroy_balloonConfig (config->balloon_configs[i]);
+			   free( config->balloon_configs );
+			}
+	    	if( config->more_stuff )
+				DestroyFreeStorage (&config->more_stuff);
+		}
+		config->type = 0 ; /* make it invalid type */
+		config->class = NULL ; /* make it invalid type */
+		config->style_defs = NULL;
+    	config->balloon_configs = NULL;
+    	config->more_stuff = NULL;
+	}
+}
+
+#define ASModuleConfig_HandleDisabled 	(0x01<<0)
+#define ASModuleConfig_DiscardDisabled 	(0x01<<1)
+#define ASModuleConfig_HandleMyStyles 	(0x01<<2)
+#define ASModuleConfig_HandleBalloons 	(0x01<<3)
+#define ASModuleConfig_HandleDefaults 	(0x01<<4)
+#define ASModuleConfig_HandleFlags	   	(0x01<<5)
+#define ASModuleConfig_HandleScalars   	(0x01<<6)
+
+#define ASModuleConfig_HandleEverything	(ASModuleConfig_HandleDisabled| \
+										 ASModuleConfig_DiscardDisabled| \
+										 ASModuleConfig_HandleMyStyles| \
+										 ASModuleConfig_HandleBalloons| \
+										 ASModuleConfig_HandleDefaults| \
+										 ASModuleConfig_HandleFlags| \
+										 ASModuleConfig_HandleScalars )
+
+#define ASModuleConfig_HandleAllButDefaults	(ASModuleConfig_DiscardDisabled| \
+										 ASModuleConfig_HandleMyStyles| \
+										 ASModuleConfig_HandleBalloons| \
+										 ASModuleConfig_HandleDefaults| \
+										 ASModuleConfig_HandleFlags| \
+										 ASModuleConfig_HandleScalars )
+
+ASModuleConfig* 
+create_ASModule_config( ASModuleConfigClass *class ) 
+{
+	ASModuleConfig* config = NULL ; 
+	
+	if( class && class->config_struct_size > 0 && class->type != 0 ) 
+	{
+		config = safecalloc( 1, class->config_struct_size );
+		init_asmodule_config( config, False );
+		config->type = class->type ;
+		config->class = class ; 
+		if( class->balloon_types != NULL ) 
+		{
+			int count = 0; 
+			while( class->balloon_types[count] > 0 ) ++count;
+			if( count > 0 ) 
+				config->balloon_configs = safecalloc( count+1, sizeof(struct balloonConfig*) ); 
+		}
+		if( class->init_config_func )
+			class->init_config_func(config, False);
+	}
+	return config ;
+}
+
+void 
+destroy_ASModule_config( ASModuleConfig *config )
+{
+	if( config ) 
+	{
+		ASModuleConfigClass *class = config->class ;
+		if( class && class->init_config_func )
+			class->init_config_func( config, True );
+		init_asmodule_config( config, True );
+		free (config);
+	}
+}
+
+MyStyleDefinition*
+free_storage_elem2MyStyleDefinition (FreeStorageElem *fs, const char *default_name)
+{
+	MyStyleDefinition *msd = NULL;
+	
+	if (fs == NULL)
+		return NULL;
+
+	msd = safecalloc (1, sizeof (MyStyleDefinition));
+	msd->type = CONFIG_MyStyles_ID;
+	
+	if (fs->sub)
+		fs = fs->sub;
+	for ( ;fs ; fs = fs->next)
+	{
+		TermDef *T = fs->term ;
+		
+		if (T->id < MYSTYLE_ID_START || T->id > MYSTYLE_ID_END) /* just in case */
+			continue;
+
+		if (T->struct_field_offset != 0) 
+		{
+			ReadConfigItemToStruct( msd, offsetof(MyStyleDefinition,set_flags), fs  );
+/* fprintf (stderr, "$$$$ name = %s, offset=%d,keyword =%s,flags_on=0x%X, set_flags = 0x%X\n",msd->Name, offsetof(MyStyleDefinition,set_flags), T->keyword, T->flags_on, msd->set_flags); */
+		}else if( T->type == TT_FLAG)
+		{
+			ReadFlagItemAuto (msd, offsetof(MyStyleDefinition,set_flags), fs, MyStyleFlags);	
+		}else
+			HandleMyStyleSpecialTerm (msd, fs);
+	}	
+	
+	if (msd->Name == NULL && default_name)
+		msd->Name = mystrdup (default_name);
+	
+	if (!CheckMyStyleDefinitionSanity (msd))
+		DestroyMyStyleDefinitions (&msd);
+		
+	return msd;
+}
+
+FreeStorageElem *
+MyStyleDefinition2free_storage_elem (MyStyleDefinition *msd, SyntaxDef *syntax)
+{
+	FreeStorageElem *fs = NULL;
+
+	if (msd == NULL || syntax == NULL)
+		return NULL;
+
+	if (msd->Name == NULL)
+		return NULL;
+
+	QuotedString2FreeStorage (syntax, &fs, NULL, msd->Name, MYSTYLE_MyStyle_ID);
+	if (fs)
+	{
+		FreeStorageElem **tail = &(fs->sub);
+		ASFlagType auto_handled = 0, auto_handled_flags = 0;
+		TermDef *T = fs->term;
+		
+		fs->sub = StructToFreeStorage (msd, offsetof (MyStyleDefinition, set_flags), T->sub_syntax, &auto_handled);
+		ADVANCE_LINKED_LIST_TAIL(tail);
+
+		/* now we need to handle some flags : */
+		*tail = StructFlags2FreeStorage (msd, offsetof(MyStyleDefinition,set_flags), T->sub_syntax, MyStyleFlags, &auto_handled_flags);
+		ADVANCE_LINKED_LIST_TAIL(tail);
+
+		/* now we need to handle some special items : */
+		*tail = MyStyleSpecialTerms2FreeStorage (msd, T->sub_syntax);
+	}		
+	return fs;
+}
+
+
+MyStyleDefinition*
+free_storage2MyStyleDefinitionsList (FreeStorageElem *fs)
+{
+	MyStyleDefinition *head = NULL;
+	MyStyleDefinition **tail = &head;
+	
+    for (; fs; fs = fs->next)
+	{
+		if (fs->term->id == MYSTYLE_MyStyle_ID)
+		{
+			*tail = free_storage_elem2MyStyleDefinition (fs->sub, NULL);
+			ADVANCE_LINKED_LIST_TAIL(tail);
+		}	
+	}
+	return head;	
+}
+
+FreeStorageElem *
+MyStyleDefinitionsList2free_storage (MyStyleDefinition *msd, SyntaxDef *syntax)
+{
+	FreeStorageElem *head = NULL;
+	FreeStorageElem **tail = &head;
+	
+    for (; msd; msd = msd->next)
+	{
+		*tail = MyStyleDefinition2free_storage_elem (msd, syntax);
+		ADVANCE_LINKED_LIST_TAIL(tail);
+	}
+	return head;	
+}
+
+
+ASModuleConfig* 
+free_storage2ASModule_config( ASModuleConfigClass *class, ASModuleConfig *config, FreeStorageElem *Storage, ASFlagType flags )
+{
+	if( Storage != NULL ) 
+	{
+		MyStyleDefinition **styles_tail = NULL;
+		FreeStorageElem **more_stuff_tail, *pCurr ;
+		
+		if( config == NULL ) 
+			config = create_ASModule_config(class);
+		
+		if( get_flags( flags, ASModuleConfig_HandleDisabled ) )
+		{
+			more_stuff_tail = &(config->more_stuff);
+			while( *more_stuff_tail ) more_stuff_tail = &(*more_stuff_tail)->next ;
+			StorageCleanUp (&Storage, more_stuff_tail, CF_DISABLED_OPTION);
+		}else if( get_flags( flags, ASModuleConfig_DiscardDisabled ) )
+		{
+			FreeStorageElem *tmp = NULL ; 
+			StorageCleanUp (&Storage, &tmp, CF_DISABLED_OPTION);
+			if( tmp )
+				DestroyFreeStorage (&tmp);
+		}
+		
+		if( get_flags( flags, ASModuleConfig_HandleMyStyles ) )
+		{
+			styles_tail = &(config->style_defs);
+			while( *styles_tail ) styles_tail = &(*styles_tail)->next ;
+		}
+	
+		if( get_flags( flags, ASModuleConfig_HandleBalloons ) && class != NULL )
+		{
+			int i = 0; 
+			while( class->balloon_types[i] > 0 )
+			{			
+				LOCAL_DEBUG_OUT( "handling balloons type %d, base_ID = %d", i, class->balloon_types[i] );
+	    		config->balloon_configs[i] = Process_balloonOptions(Storage, config->balloon_configs[i], class->balloon_types[i]);
+				++i ;
+			}
+		}
+
+	    for (pCurr = Storage; pCurr; pCurr = pCurr->next)
+		{
+			TermDef *T = pCurr->term ;
+			if ( T != NULL)
+			{
+				if( T->id == MODULE_Defaults_ID && class != NULL ) 
+				{
+					if( pCurr->sub && get_flags( flags, ASModuleConfig_HandleDefaults )) 
+					{
+						ASModuleConfig *defaults = free_storage2ASModule_config( class, NULL, pCurr->sub, 
+																					ASModuleConfig_DiscardDisabled|
+																					ASModuleConfig_HandleFlags|
+																					ASModuleConfig_HandleScalars| 
+																					ASModuleConfig_HandleBalloons );
+						if( defaults )
+						{								
+							class->free_storage2config_func( defaults, pCurr->sub );
+							class->merge_config_func( defaults, config );				
+							/* our stuff does not get merged - handle manually : */
+							if( defaults->style_defs ) 
+								DestroyMyStyleDefinitions ( &(defaults->style_defs) );
+							defaults->style_defs = config->style_defs ; 
+						
+							MergeBalloonOptions ( defaults, config);
+					
+							destroy_ASModule_config( config );
+							config = defaults ;
+						}
+					}				
+				}else if( T->id == MYSTYLE_MyStyle_ID && 
+				          get_flags( flags, ASModuleConfig_HandleMyStyles ) )
+				{
+             		*styles_tail = free_storage_elem2MyStyleDefinition (pCurr->sub, NULL);
+					if (*styles_tail)
+						styles_tail = &((*styles_tail)->next); 
+				}else if( T->struct_field_offset != 0  && 
+				         get_flags( flags, ASModuleConfig_HandleScalars )) 
+				{
+					ReadConfigItemToStruct( config, class->set_flags_field_offset, pCurr  );
+				}else if( T->type == TT_FLAG && class && class->flags_xref != NULL && 
+				         get_flags( flags, ASModuleConfig_HandleFlags ))
+				{
+					ReadFlagItemAuto (config, class->set_flags_field_offset, pCurr, class->flags_xref);	
+				}
+			}
+		}
+	}
+	return config;
+}
+
+void
+merge_ASModule_config( ASModuleConfigClass *class, ASModuleConfig *to, ASModuleConfig *from )
+{
+	MergeBalloonOptions ( to, from);
+	/* TODO: also need to merge MyStyle defs */
+
+	if( class->merge_config_func ) 
+		class->merge_config_func( to, from );
+
+	if( class->set_flags_field_offset > 0 ) 
+	{
+		ASFlagType *set_flags_ptr_to = (void*)to + class->set_flags_field_offset ;
+		ASFlagType *set_flags_ptr_from = (void*)from + class->set_flags_field_offset ;
+		
+		set_flags( *set_flags_ptr_to, *set_flags_ptr_from );
+	}
+}
+
+
+static ASModuleConfig *
+parse_asmodule_config_file( ASModuleConfigClass *class, ASModuleConfig *config, const char *filename, SyntaxDef *syntax, ASFlagType parser_flags, ASFlagType handling_flags )
+{
+	ConfigData    cd ;
+	ConfigDef    *ConfigReader;
+
+	cd.filename = filename ;
+	ConfigReader = InitConfigReader (MyName, syntax, CDT_Filename, cd, NULL);
+	if ( ConfigReader != NULL )
+	{
+		FreeStorageElem *Storage = NULL;
+		set_flags( ConfigReader->flags, parser_flags );
+		ParseConfig (ConfigReader, &Storage);
+		config = free_storage2ASModule_config( class, config, Storage, handling_flags );
+		class->free_storage2config_func( config, Storage );
+ 		DestroyFreeStorage (&Storage);
+		DestroyConfig (ConfigReader);
+        show_progress("configuration loaded from \"%s\" ...", filename);
+	}
+
+/*    PrintMyStyleDefinitions( config->style_defs ); */
+	return config;
+}
+
+ASModuleConfig *
+parse_asmodule_look(ASModuleConfigClass *class, ASModuleConfig *module_config )
+{
+	const char *const_configfile =  get_session_file (Session, 0, F_CHANGE_LOOK, False);
+    if( const_configfile != NULL && class->look_syntax != NULL )
+	{
+		ASFlagType parser_flags 	= 	CP_IgnoreForeign ; 
+		ASFlagType handling_flags 	= 	ASModuleConfig_DiscardDisabled|	
+										ASModuleConfig_HandleBalloons| 
+										ASModuleConfig_HandleFlags|
+										ASModuleConfig_HandleScalars ;
+		if( !get_flags(class->flags, ASMC_HandlePublicLookOptions) )
+			set_flags( parser_flags, CP_IgnorePublic);
+		if( get_flags(class->flags,ASMC_HandleLookMyStyles) ) 
+			set_flags(handling_flags, ASModuleConfig_HandleMyStyles);	
+		return parse_asmodule_config_file(  class, module_config, const_configfile, class->look_syntax, 
+											parser_flags, handling_flags );
+	}
+	return module_config;
+}
+
+ASModuleConfig *
+parse_asmodule_feel(ASModuleConfigClass *class, ASModuleConfig *module_config )
+{
+	const char *const_configfile =  get_session_file (Session, 0, F_CHANGE_FEEL, False);
+    if( const_configfile != NULL && class->feel_syntax != NULL )
+	{
+		ASFlagType parser_flags 	= 	CP_IgnoreForeign ; 
+		ASFlagType handling_flags 	= 	ASModuleConfig_DiscardDisabled|	
+										ASModuleConfig_HandleBalloons| 
+										ASModuleConfig_HandleFlags|
+										ASModuleConfig_HandleScalars ;
+		if( !get_flags(class->flags, ASMC_HandlePublicFeelOptions) )
+			set_flags( parser_flags, CP_IgnorePublic);
+		return parse_asmodule_config_file( class, module_config, const_configfile, class->feel_syntax, 
+											parser_flags, handling_flags );
+	}
+	return module_config;
+}
+
+ASModuleConfig *
+parse_asmodule_private_config(ASModuleConfigClass *class, ASModuleConfig *module_config )
+{
+	if( class->private_config_file && class->module_syntax )
+	{
+    	char *configfile = make_session_file(Session, class->private_config_file, False );
+        if( configfile != NULL )
+			return parse_asmodule_config_file( class, module_config, configfile, class->module_syntax, 
+											   CP_IgnoreForeign, 
+											    ASModuleConfig_DiscardDisabled|
+												ASModuleConfig_HandleMyStyles| 
+										 		ASModuleConfig_HandleBalloons| 
+										 		ASModuleConfig_HandleDefaults| 
+										 		ASModuleConfig_HandleFlags|
+										 		ASModuleConfig_HandleScalars);
+	}
+	return module_config;
+}
+
+
+ASModuleConfig *
+parse_asmodule_config_all(ASModuleConfigClass *class )
+{
+	ASModuleConfig *config = NULL;
+	LOCAL_DEBUG_CALLER_OUT( "class = %p, private_file = \"%s\"",class, class?class->private_config_file:"(none)" ); 
+	config = parse_asmodule_look(class, config );
+	config = parse_asmodule_feel(class, config );
+	config = parse_asmodule_private_config(class, config );
+
+	return config;
+}
+
+/******************************************************************************
+ * Test cases and utilities
+ ******************************************************************************/
+
